package jdk.internal.org.objectweb.asm.commons;

import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Type;
import jdk.internal.org.objectweb.asm.Opcodes;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Map;
import jdk.internal.org.objectweb.asm.Label;
import java.util.List;
import jdk.internal.org.objectweb.asm.MethodVisitor;

public class AnalyzerAdapter extends MethodVisitor
{
    public List<Object> locals;
    public List<Object> stack;
    private List<Label> labels;
    public Map<Object, Object> uninitializedTypes;
    private int maxStack;
    private int maxLocals;
    private String owner;
    
    public AnalyzerAdapter(String s, int n, String s2, String s3, MethodVisitor methodVisitor) {
        this(327680, s, n, s2, s3, methodVisitor);
        if (this.getClass() != AnalyzerAdapter.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    protected AnalyzerAdapter(int api, String owner, int n, String anObject, String methodDescriptor, MethodVisitor methodVisitor) {
        super(api, methodVisitor);
        this.owner = owner;
        this.locals = new ArrayList();
        this.stack = new ArrayList();
        this.uninitializedTypes = new HashMap();
        if ((n & 0x8) == 0x0) {
            if ("<init>".equals(anObject)) {
                this.locals.add(Opcodes.UNINITIALIZED_THIS);
            }
            else {
                this.locals.add(owner);
            }
        }
        Type[] argumentTypes = Type.getArgumentTypes(methodDescriptor);
        for (int i = 0; i < argumentTypes.length; ++i) {
            switch (argumentTypes[i].getSort()) {
                case 1:
                case 2:
                case 3:
                case 4:
                case 5: {
                    this.locals.add(Opcodes.INTEGER);
                    break;
                }
                case 6: {
                    this.locals.add(Opcodes.FLOAT);
                    break;
                }
                case 7: {
                    this.locals.add(Opcodes.LONG);
                    this.locals.add(Opcodes.TOP);
                    break;
                }
                case 8: {
                    this.locals.add(Opcodes.DOUBLE);
                    this.locals.add(Opcodes.TOP);
                    break;
                }
                case 9: {
                    this.locals.add(argumentTypes[i].getDescriptor());
                    break;
                }
                default: {
                    this.locals.add(argumentTypes[i].getInternalName());
                    break;
                }
            }
        }
        this.maxLocals = this.locals.size();
    }
    
    @Override
    public void visitFrame(int type, int numLocal, Object[] local, int numStack, Object[] stack) {
        if (type != -1) {
            throw new IllegalStateException("ClassReader.accept() should be called with EXPAND_FRAMES flag");
        }
        if (this.mv != null) {
            this.mv.visitFrame(type, numLocal, local, numStack, stack);
        }
        if (this.locals != null) {
            this.locals.clear();
            this.stack.clear();
        }
        else {
            this.locals = new ArrayList();
            this.stack = new ArrayList();
        }
        visitFrameTypes(numLocal, local, this.locals);
        visitFrameTypes(numStack, stack, this.stack);
        this.maxStack = Math.max(this.maxStack, this.stack.size());
        return;
    }
    
    private static void visitFrameTypes(int n, Object[] array, List<Object> list) {
        for (Object o : array) {
            list.add(o);
            if (o == Opcodes.LONG || o == Opcodes.DOUBLE) {
                list.add(Opcodes.TOP);
            }
        }
    }
    
    @Override
    public void visitInsn(int opcode) {
        if (this.mv != null) {
            this.mv.visitInsn(opcode);
        }
        this.execute(opcode, 0, null);
        if ((opcode >= 172 && opcode <= 177) || opcode == 191) {
            this.locals = null;
            this.stack = null;
        }
    }
    
    @Override
    public void visitIntInsn(int opcode, int operand) {
        if (this.mv != null) {
            this.mv.visitIntInsn(opcode, operand);
        }
        this.execute(opcode, operand, null);
    }
    
    @Override
    public void visitVarInsn(int opcode, int varIndex) {
        if (this.mv != null) {
            this.mv.visitVarInsn(opcode, varIndex);
        }
        this.execute(opcode, varIndex, null);
    }
    
    @Override
    public void visitTypeInsn(int opcode, String type) {
        if (opcode == 187) {
            if (this.labels == null) {
                Label label = new Label();
                (this.labels = new ArrayList(3)).add(label);
                if (this.mv != null) {
                    this.mv.visitLabel(label);
                }
            }
            for (int i = 0; i < this.labels.size(); ++i) {
                this.uninitializedTypes.put(this.labels.get(i), type);
            }
        }
        if (this.mv != null) {
            this.mv.visitTypeInsn(opcode, type);
        }
        this.execute(opcode, 0, type);
    }
    
    @Override
    public void visitFieldInsn(int opcode, String owner, String name, String descriptor) {
        if (this.mv != null) {
            this.mv.visitFieldInsn(opcode, owner, name, descriptor);
        }
        this.execute(opcode, 0, descriptor);
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, opcode == 185);
        return;
    }
    
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor, isInterface);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, isInterface);
        return;
    }
    
    private void doVisitMethodInsn(int opcode, String owner, String name, String descriptor, boolean isInterface) {
        if (this.mv != null) {
            this.mv.visitMethodInsn(opcode, owner, name, descriptor, isInterface);
        }
        if (this.locals == null) {
            this.labels = null;
            return;
        }
        this.pop(descriptor);
        if (opcode != 184) {
            Object pop = this.pop();
            if (opcode == 183 && name.charAt(0) == '<') {
                String s;
                if (pop == Opcodes.UNINITIALIZED_THIS) {
                    s = this.owner;
                }
                else {
                    s = this.uninitializedTypes.get(pop);
                }
                for (int i = 0; i < this.locals.size(); ++i) {
                    if (this.locals.get(i) == pop) {
                        this.locals.set(i, s);
                    }
                }
                for (int j = 0; j < this.stack.size(); ++j) {
                    if (this.stack.get(j) == pop) {
                        this.stack.set(j, s);
                    }
                }
            }
        }
        this.pushDesc(descriptor);
        this.labels = null;
        return;
    }
    
    @Override
    public void visitInvokeDynamicInsn(String name, String descriptor, Handle bootstrapMethodHandle, Object... bootstrapMethodArguments) {
        if (this.mv != null) {
            this.mv.visitInvokeDynamicInsn(name, descriptor, bootstrapMethodHandle, bootstrapMethodArguments);
        }
        if (this.locals == null) {
            this.labels = null;
            return;
        }
        this.pop(descriptor);
        this.pushDesc(descriptor);
        this.labels = null;
        return;
    }
    
    @Override
    public void visitJumpInsn(int opcode, Label label) {
        if (this.mv != null) {
            this.mv.visitJumpInsn(opcode, label);
        }
        this.execute(opcode, 0, null);
        if (opcode == 167) {
            this.locals = null;
            this.stack = null;
        }
    }
    
    @Override
    public void visitLabel(Label label) {
        if (this.mv != null) {
            this.mv.visitLabel(label);
        }
        if (this.labels == null) {
            this.labels = new ArrayList(3);
        }
        this.labels.add(label);
    }
    
    @Override
    public void visitLdcInsn(Object value) {
        if (this.mv != null) {
            this.mv.visitLdcInsn(value);
        }
        if (this.locals == null) {
            this.labels = null;
            return;
        }
        if ((value instanceof Integer)) {
            this.push(Opcodes.INTEGER);
        }
        else if ((value instanceof Long)) {
            this.push(Opcodes.LONG);
            this.push(Opcodes.TOP);
        }
        else if ((value instanceof Float)) {
            this.push(Opcodes.FLOAT);
        }
        else if ((value instanceof Double)) {
            this.push(Opcodes.DOUBLE);
            this.push(Opcodes.TOP);
        }
        else if ((value instanceof String)) {
            this.push("java/lang/String");
        }
        else if ((value instanceof Type)) {
            int sort = ((Type)value).getSort();
            if (sort == 10 || sort == 9) {
                this.push("java/lang/Class");
            }
            else if (sort == 11) {
                this.push("java/lang/invoke/MethodType");
            }
            else {
                throw new IllegalArgumentException();
            }
        }
        else if ((value instanceof Handle)) {
            this.push("java/lang/invoke/MethodHandle");
        }
        else {
            throw new IllegalArgumentException();
        }
        this.labels = null;
        return;
    }
    
    @Override
    public void visitIincInsn(int varIndex, int increment) {
        if (this.mv != null) {
            this.mv.visitIincInsn(varIndex, increment);
        }
        this.execute(132, varIndex, null);
    }
    
    @Override
    public void visitTableSwitchInsn(int min, int max, Label dflt, Label... labels) {
        if (this.mv != null) {
            this.mv.visitTableSwitchInsn(min, max, dflt, labels);
        }
        this.execute(170, 0, null);
        this.locals = null;
        this.stack = null;
    }
    
    @Override
    public void visitLookupSwitchInsn(Label dflt, int[] keys, Label[] labels) {
        if (this.mv != null) {
            this.mv.visitLookupSwitchInsn(dflt, keys, labels);
        }
        this.execute(171, 0, null);
        this.locals = null;
        this.stack = null;
    }
    
    @Override
    public void visitMultiANewArrayInsn(String descriptor, int numDimensions) {
        if (this.mv != null) {
            this.mv.visitMultiANewArrayInsn(descriptor, numDimensions);
        }
        this.execute(197, numDimensions, descriptor);
    }
    
    @Override
    public void visitMaxs(int b, int b2) {
        if (this.mv != null) {
            this.maxStack = Math.max(this.maxStack, b);
            this.maxLocals = Math.max(this.maxLocals, b2);
            this.mv.visitMaxs(this.maxStack, this.maxLocals);
        }
    }
    
    private Object get(int n) {
        this.maxLocals = Math.max(this.maxLocals, n + 1);
        return (n < this.locals.size()) ? this.locals.get(n) : Opcodes.TOP;
    }
    
    private void set(int i, Object o) {
        this.maxLocals = Math.max(this.maxLocals, i + 1);
        while (i >= this.locals.size()) {
            this.locals.add(Opcodes.TOP);
        }
        this.locals.set(i, o);
    }
    
    private void push(Object o) {
        this.stack.add(o);
        this.maxStack = Math.max(this.maxStack, this.stack.size());
    }
    
    private void pushDesc(String s) {
        int n = (s.charAt(0) == '(') ? (s.indexOf(41) + 1) : 0;
        switch (s.charAt(n)) {
            case 'V': {
                return;
            }
            case 'B':
            case 'C':
            case 'I':
            case 'S':
            case 'Z': {
                this.push(Opcodes.INTEGER);
                return;
            }
            case 'F': {
                this.push(Opcodes.FLOAT);
                return;
            }
            case 'J': {
                this.push(Opcodes.LONG);
                this.push(Opcodes.TOP);
                return;
            }
            case 'D': {
                this.push(Opcodes.DOUBLE);
                this.push(Opcodes.TOP);
                return;
            }
            case '[': {
                if (n == 0) {
                    this.push(s);
                    break;
                }
                this.push(s.substring(n, s.length()));
                break;
            }
            default: {
                if (n == 0) {
                    this.push(s.substring(1, s.length() - 1));
                    break;
                }
                this.push(s.substring(n + 1, s.length() - 1));
                break;
            }
        }
    }
    
    private Object pop() {
        return this.stack.remove(this.stack.size() - 1);
    }
    
    private void pop(int n) {
        int size = this.stack.size();
        for (int n2 = size - n, i = size - 1; i >= n2; --i) {
            this.stack.remove(i);
        }
    }
    
    private void pop(String methodDescriptor) {
        int char1 = methodDescriptor.charAt(0);
        if (char1 == '(') {
            int n = 0;
            Type[] argumentTypes = Type.getArgumentTypes(methodDescriptor);
            for (int i = 0; i < argumentTypes.length; ++i) {
                n += argumentTypes[i].getSize();
            }
            this.pop(n);
        }
        else if (char1 == 'J' || char1 == 'D') {
            this.pop(2);
        }
        else {
            this.pop(1);
        }
    }
    
    private void execute(int n, int n2, String s) {
        if (this.locals == null) {
            this.labels = null;
            return;
        }
        switch (n) {
            case 0:
            case 116:
            case 117:
            case 118:
            case 119:
            case 145:
            case 146:
            case 147:
            case 167:
            case 177: {
                break;
            }
            case 1: {
                this.push(Opcodes.NULL);
                break;
            }
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
            case 7:
            case 8:
            case 16:
            case 17: {
                this.push(Opcodes.INTEGER);
                break;
            }
            case 9:
            case 10: {
                this.push(Opcodes.LONG);
                this.push(Opcodes.TOP);
                break;
            }
            case 11:
            case 12:
            case 13: {
                this.push(Opcodes.FLOAT);
                break;
            }
            case 14:
            case 15: {
                this.push(Opcodes.DOUBLE);
                this.push(Opcodes.TOP);
                break;
            }
            case 21:
            case 23:
            case 25: {
                this.push(this.get(n2));
                break;
            }
            case 22:
            case 24: {
                this.push(this.get(n2));
                this.push(Opcodes.TOP);
                break;
            }
            case 46:
            case 51:
            case 52:
            case 53: {
                this.pop(2);
                this.push(Opcodes.INTEGER);
                break;
            }
            case 47:
            case 143: {
                this.pop(2);
                this.push(Opcodes.LONG);
                this.push(Opcodes.TOP);
                break;
            }
            case 48: {
                this.pop(2);
                this.push(Opcodes.FLOAT);
                break;
            }
            case 49:
            case 138: {
                this.pop(2);
                this.push(Opcodes.DOUBLE);
                this.push(Opcodes.TOP);
                break;
            }
            case 50: {
                this.pop(1);
                Object pop = this.pop();
                if ((pop instanceof String)) {
                    this.pushDesc(((String)pop).substring(1));
                    break;
                }
                this.push("java/lang/Object");
                break;
            }
            case 54:
            case 56:
            case 58: {
                this.set(n2, this.pop());
                if (n2 > 0) {
                    Object value = this.get(n2 - 1);
                    if (value == Opcodes.LONG || value == Opcodes.DOUBLE) {
                        this.set(n2 - 1, Opcodes.TOP);
                        break;
                    }
                    break;
                }
                else {
                    break;
                }
            }
            case 55:
            case 57: {
                this.pop(1);
                this.set(n2, this.pop());
                this.set(n2 + 1, Opcodes.TOP);
                if (n2 > 0) {
                    Object value2 = this.get(n2 - 1);
                    if (value2 == Opcodes.LONG || value2 == Opcodes.DOUBLE) {
                        this.set(n2 - 1, Opcodes.TOP);
                        break;
                    }
                    break;
                }
                else {
                    break;
                }
            }
            case 79:
            case 81:
            case 83:
            case 84:
            case 85:
            case 86: {
                this.pop(3);
                break;
            }
            case 80:
            case 82: {
                this.pop(4);
                break;
            }
            case 87:
            case 153:
            case 154:
            case 155:
            case 156:
            case 157:
            case 158:
            case 170:
            case 171:
            case 172:
            case 174:
            case 176:
            case 191:
            case 194:
            case 195:
            case 198:
            case 199: {
                this.pop(1);
                break;
            }
            case 88:
            case 159:
            case 160:
            case 161:
            case 162:
            case 163:
            case 164:
            case 165:
            case 166:
            case 173:
            case 175: {
                this.pop(2);
                break;
            }
            case 89: {
                Object pop2 = this.pop();
                this.push(pop2);
                this.push(pop2);
                break;
            }
            case 90: {
                Object pop3 = this.pop();
                Object pop4 = this.pop();
                this.push(pop3);
                this.push(pop4);
                this.push(pop3);
                break;
            }
            case 91: {
                Object pop5 = this.pop();
                Object pop6 = this.pop();
                Object pop7 = this.pop();
                this.push(pop5);
                this.push(pop7);
                this.push(pop6);
                this.push(pop5);
                break;
            }
            case 92: {
                Object pop8 = this.pop();
                Object pop9 = this.pop();
                this.push(pop9);
                this.push(pop8);
                this.push(pop9);
                this.push(pop8);
                break;
            }
            case 93: {
                Object pop10 = this.pop();
                Object pop11 = this.pop();
                Object pop12 = this.pop();
                this.push(pop11);
                this.push(pop10);
                this.push(pop12);
                this.push(pop11);
                this.push(pop10);
                break;
            }
            case 94: {
                Object pop13 = this.pop();
                Object pop14 = this.pop();
                Object pop15 = this.pop();
                Object pop16 = this.pop();
                this.push(pop14);
                this.push(pop13);
                this.push(pop16);
                this.push(pop15);
                this.push(pop14);
                this.push(pop13);
                break;
            }
            case 95: {
                Object pop17 = this.pop();
                Object pop18 = this.pop();
                this.push(pop17);
                this.push(pop18);
                break;
            }
            case 96:
            case 100:
            case 104:
            case 108:
            case 112:
            case 120:
            case 122:
            case 124:
            case 126:
            case 128:
            case 130:
            case 136:
            case 142:
            case 149:
            case 150: {
                this.pop(2);
                this.push(Opcodes.INTEGER);
                break;
            }
            case 97:
            case 101:
            case 105:
            case 109:
            case 113:
            case 127:
            case 129:
            case 131: {
                this.pop(4);
                this.push(Opcodes.LONG);
                this.push(Opcodes.TOP);
                break;
            }
            case 98:
            case 102:
            case 106:
            case 110:
            case 114:
            case 137:
            case 144: {
                this.pop(2);
                this.push(Opcodes.FLOAT);
                break;
            }
            case 99:
            case 103:
            case 107:
            case 111:
            case 115: {
                this.pop(4);
                this.push(Opcodes.DOUBLE);
                this.push(Opcodes.TOP);
                break;
            }
            case 121:
            case 123:
            case 125: {
                this.pop(3);
                this.push(Opcodes.LONG);
                this.push(Opcodes.TOP);
                break;
            }
            case 132: {
                this.set(n2, Opcodes.INTEGER);
                break;
            }
            case 133:
            case 140: {
                this.pop(1);
                this.push(Opcodes.LONG);
                this.push(Opcodes.TOP);
                break;
            }
            case 134: {
                this.pop(1);
                this.push(Opcodes.FLOAT);
                break;
            }
            case 135:
            case 141: {
                this.pop(1);
                this.push(Opcodes.DOUBLE);
                this.push(Opcodes.TOP);
                break;
            }
            case 139:
            case 190:
            case 193: {
                this.pop(1);
                this.push(Opcodes.INTEGER);
                break;
            }
            case 148:
            case 151:
            case 152: {
                this.pop(4);
                this.push(Opcodes.INTEGER);
                break;
            }
            case 168:
            case 169: {
                throw new RuntimeException("JSR/RET are not supported");
            }
            case 178: {
                this.pushDesc(s);
                break;
            }
            case 179: {
                this.pop(s);
                break;
            }
            case 180: {
                this.pop(1);
                this.pushDesc(s);
                break;
            }
            case 181: {
                this.pop(s);
                this.pop();
                break;
            }
            case 187: {
                this.push(this.labels.get(0));
                break;
            }
            case 188: {
                this.pop();
                switch (n2) {
                    case 4: {
                        this.pushDesc("[Z");
                        break;
                    }
                    case 5: {
                        this.pushDesc("[C");
                        break;
                    }
                    case 8: {
                        this.pushDesc("[B");
                        break;
                    }
                    case 9: {
                        this.pushDesc("[S");
                        break;
                    }
                    case 10: {
                        this.pushDesc("[I");
                        break;
                    }
                    case 6: {
                        this.pushDesc("[F");
                        break;
                    }
                    case 7: {
                        this.pushDesc("[D");
                        break;
                    }
                    default: {
                        this.pushDesc("[J");
                        break;
                    }
                }
                break;
            }
            case 189: {
                this.pop();
                this.pushDesc("[" + Type.getObjectType(s));
                break;
            }
            case 192: {
                this.pop();
                this.pushDesc(Type.getObjectType(s).getDescriptor());
                break;
            }
            default: {
                this.pop(n2);
                this.pushDesc(s);
                break;
            }
        }
        this.labels = null;
        return;
    }
}
